import { EditorView } from 'codemirror';
import { EditorState } from '@codemirror/state';
import { html } from '@codemirror/lang-html';
import { glsl, getBaseExtensions } from '../util/codemirror';
import '../style.scss';

export default class Editor {
  constructor(config = {}) {
    const parent: HTMLElement =
      typeof config.parent === 'string'
        ? document.querySelector(config.parent)
        : config.parent || (document.body as HTMLElement);

    const { doc, uniforms, onUpdate, language } = {
      doc: '',
      uniforms: [],
      onUpdate: (e: any) => e,
      language: 'GLSL',
      ...config,
    };

    this.language = language;
    this.onUpdate = onUpdate.bind(this);
    this.parent = parent;
    this.doc = doc || '';
    this.uniforms = uniforms;
    this.state = this.createState();
    this.view = new EditorView({
      parent: this.parent,
      state: this.state,
    });
  }

  getLanguage(): any {
    switch (this.language) {
      case 'GLSL':
        return glsl(this.uniforms);
      case 'HTML':
        return html();
    }
  }

  createState() {
    return EditorState.create({
      doc: this.doc.trim(),
      extensions: [
        ...getBaseExtensions({ parent: this.parent }),
        this.getLanguage(),
        EditorView.updateListener.of(this.update.bind(this)),
      ],
    });
  }

  update(e) {
    if (typeof this.onUpdate !== 'function') return;
    const { state, docChanged } = e;
    if (!docChanged) return;
    this.onUpdate(state.doc.toString());
  }
}
