import { Shader, Editor } from './';
import { DEFAULT_FRAGMENT_SHADER } from './../constants/glsl-defaults';
import { type Uniform } from '../types/uniforms';
import { type ShaderConfig } from './Shader';
import '../style.scss';

export default class Sandbox {
  public shader: string;
  public uniforms: Uniform[];
  public parent: HTMLElement | null;
  public instances: {
    shader: Shader;
    editor: Editor;
  };
  private raf: any;
  public animate: boolean;

  constructor(config: ShaderConfig = {}) {
    const parent: HTMLElement = (
      typeof config.parent === 'string'
        ? document.querySelector(config.parent)
        : config.parent || (document.body as HTMLElement)
    ) as HTMLElement;

    const { shader, uniforms, animate } = {
      shader: DEFAULT_FRAGMENT_SHADER,
      uniforms: [],
      animate: true,
      ...(config || {}),
    };

    this.raf;
    this.shader = shader;
    this.uniforms = uniforms;
    this.parent = parent;
    this.animate = animate;

    const shaderConfig: ShaderConfig = {
      fragmentShader: this.shader,
      parent: this.parent,
      dpr: Math.max(1, window.devicePixelRatio / 2),
      uniforms: this.uniforms,
      animate: this.animate,
      shaderpad: config.shaderpad,
    };

    const editorConfig = {
      doc: this.shader,
      uniforms: this.uniforms,
      onUpdate: this.onUpdate.bind(this),
    };

    this.instances = {
      shader: new Shader(shaderConfig),
      editor: new Editor(editorConfig),
    };
  }

  onUpdate(fragmentShader: string) {
    this.instances.shader.rebuild({ fragmentShader, uniforms: this.uniforms });
  }

  set uniform(val) {
    this.instances.shader.uniform = val;
  }
}
